import { reducedInfo, validBlocks } from "../lib/space/validBlocks"
import { world, system, Player } from "@minecraft/server"
import { apiCooldown } from "../lib/player/cooldown"
import { blockInfo } from "../lib/space/areaInfo"
import { apiNumbers } from "../lib/math/numbers"
import { apiWandInfo } from "../lib/wand"

world.afterEvents.entityHitBlock.subscribe(({damagingEntity: entity, hitBlock: block, blockFace}) => {
    const player = entity as Player
    const hand = player.getComponent("equippable").getEquipment("Mainhand")
    if(!hand) return
    if(!apiWandInfo.testValidWand(hand)) return
    if(apiCooldown.testTimeOut(player)) return
    apiCooldown.setCooldown(player)
    validBlocks.reduceBlocks(player, hand, block, blockFace, true, (info: reducedInfo, player: Player) => {
        if(!info) return apiCooldown.setCooldown(player, true)
        system.runJob(spawnParticle(info.blocks))
    })
})

function* spawnParticle(blocks: blockInfo[]): Generator<void> {
    for(const info of blocks){
        info.front.dimension.spawnParticle("minecraft:shulker_bullet", apiNumbers.centerVector(info.front.location))
        yield
    }
}