import { world, system, BlockPermutation, GameMode, Player, ItemStack } from "@minecraft/server"
import { reducedInfo, validBlocks } from "../lib/space/validBlocks"
import { apiDurability } from "../lib/item/durability"
import { apiInventory } from "../lib/player/inventory"
import { apiConfigInfo } from "../lib/player/config"
import { apiCooldown } from "../lib/player/cooldown"
import { blockInfo } from "../lib/space/areaInfo"
import { apiWandInfo } from "../lib/wand"

let cooldown = new Map()

world.beforeEvents.worldInitialize.subscribe(({itemComponentRegistry: customI}) => {
    customI.registerCustomComponent("builder_wand:onUseOn", {
        onUseOn: ({block, blockFace, source, itemStack: item}) => {
            const player = source as Player
            if(cooldown.has(player.id)) return
            cooldown.set(player.id, true)
            if(apiWandInfo.testValidWand(item)){
                try{
                    system.run(() => {
                        validBlocks.reduceBlocks(player, item, block, blockFace, false, (info: reducedInfo, player: Player, item: ItemStack) => {
                            if(!info) return
                            apiCooldown.setCooldown(player as Player, true)
                            if(player.getGameMode() != GameMode.creative && !apiWandInfo.testWandCreative(item)) apiDurability.removeDurability(player, player.getComponent("equippable"), item, info.damage > info.blocks.length ? info.blocks.length : info.damage)
                            if(player.getGameMode() != GameMode.creative && !apiWandInfo.testWandCreative(item)) apiInventory.removeItems(player.getComponent("inventory").container, info.inventory, info.blocks.length)
                            system.runJob(placeBlocks(apiWandInfo.testWandCreative(item) && !apiConfigInfo.getWandConfig(player, item).connect ? info.allBlocks : info.blocks, player))
                        })
                    })
                } catch {
                    cooldown.delete(player.id)
                }
            }
        }
    })
})

function* placeBlocks(blocks: blockInfo[], player: Player): Generator<void> {
    try{
        for(const info of blocks){
            if(info.front.typeId == "minecraft:air" || info.front.typeId == "minecraft:short_grass") info.front.setPermutation(BlockPermutation.resolve(info.back.typeId, info.back.permutation.getAllStates()))
            yield
        }
        cooldown.delete(player.id)
    } catch {
        cooldown.delete(player.id)
    }
}