import { world, ItemStack, EntityEquippableComponent, GameMode, Player } from "@minecraft/server"
import { apiNumbers } from "../math/numbers"

export const apiDurability = new class apiDurability {
    getDurability(item: ItemStack, player?: Player): number {
        if(player?.getGameMode() == GameMode.creative) return 64 ** 2 ** 2
        if(!item.hasComponent("durability")) return 64 ** 2 ** 2
        const durability = item.getComponent("durability")
        return durability.maxDurability - durability.damage
    }

    getMaxDurability(item: ItemStack): number { return item.getComponent("durability").maxDurability }

    getRemoveDurability(item: ItemStack, amount: number): number {
        if(!item.hasComponent("durability")) return 0
        const enchantment = item.getComponent("enchantable")
        const chance = enchantment?.hasEnchantment("unbreaking") ? Math.floor(100-(100/(enchantment.getEnchantment("unbreaking").level +1))) : 0
        return amount - apiNumbers.clamp(Math.round(amount * ((Math.random() * (chance - (chance -10)) + (chance -10)) /100)), 0, Infinity)
    }

    removeDurability(player: Player, hand: EntityEquippableComponent, item: ItemStack, damage: number){
        if(!item.hasComponent("durability")) return
        damage = apiNumbers.clamp(damage, 0, item.getComponent("durability").maxDurability - item.getComponent("durability").damage)
        item.getComponent("durability").damage += damage
        const durability = this.getDurability(item)
        item.setLore([`Durability: ${durability}/${this.getMaxDurability(item)}`])
        hand.setEquipment("Mainhand", durability <= 0 ? undefined : item)
        if(durability <= 0) player.playSound("random.break", {location: player.location})
    }
}