import { world, ItemStack, Block, Player, Vector3 } from "@minecraft/server"
import { apiConfigInfo } from "../player/config"
import { validBlocks } from "./validBlocks"

export const apiAreaInfo = new class apiAreaInfo {
    offsetDirection: { [key: string]: offsetInfo } = { "North": ["z", -1], "South": ["z", 1], "East": ["x", 1], "West": ["x", -1], "Up": ["y", 1], "Down": ["y", -1] }
    AreaDirection: { [key: string]: areaDirection } = { "North": ["x", "y"], "South": ["x", "y"], "East": ["z", "y"], "West": ["z", "y"], "Up": ["x", "z"], "Down": ["x", "z"] }

    getBlocksToPlace(player: Player, block: Block, direction: string, item: ItemStack): blockInfo[] {
        const config = apiConfigInfo.getWandConfig(player, item)
        const size = config.size
        const area = this.getAreaPos(block.location, direction, size)
        return validBlocks.getAll(player, area, block, direction)
    }

    getAreaPos(center: Vector3, direction: string, size: number): selectedArea {
        return this.applyDirection(center, direction, Math.floor(size /2))
    }

    applyOffset(oldPos: Vector3, direction: string): Vector3 {
        const offset = this.offsetDirection[direction]
        const pos = {...oldPos}
        pos[offset[0]] += offset[1]
        return pos
    }

    applyDirection(oldPos: Vector3, direction: string, radius: number): selectedArea {
        const pos = {...oldPos}
        const offset = this.AreaDirection[direction]
        pos[offset[0]] += -1 * radius; pos[offset[1]] += -1 * radius
        const min = {...pos}
        pos[offset[0]] += 2 * radius; pos[offset[1]] += 2 * radius
        return {min: min, max: pos}
    }
}

export interface selectedArea {
    min: Vector3,
    max: Vector3
}

export interface blockInfo {
    back: Block,
    front: Block
}

interface offsetInfo extends Array<string | number> {
    0: "x" | "y" | "z",
    1: number
}

interface areaDirection extends Array<string | number> {
    0: "x" | "y" | "z",
    1: "x" | "y" | "z"
}