import { world, ItemStack, Player, Vector3 } from "@minecraft/server"
import { blockInfo } from "./space/areaInfo"
import { wandSize } from "./variables"
import { apiNumbers } from "./math/numbers"

export const apiWandInfo = new class apiWandInfo {
    getMaxSize(item: ItemStack): number {
        return wandSize[item.typeId]
    }

    getOldDamage(item: ItemStack, pos: Vector3): number | undefined {
        const oldDamage = item.getDynamicProperty(`buidler_wand:old_damage_${item.typeId.replace("builder_wand:", "").replace("builder_", "")}`)
        if(!oldDamage) return
        const info: oldDamage = JSON.parse(oldDamage as string)
        if(info.pos != apiNumbers.vectorToString(pos)) return
        return info.damage
    }

    reduceAmount(blocks: blockInfo[], durability: number, damage: number, inventory: number): blockInfo[] {
        const newArray = [...blocks]
        let amount = newArray.length
        const removeDurability = durability > damage ? amount : durability
        if(amount > removeDurability) amount = durability
        if(amount > inventory) amount = inventory
        newArray.splice(amount)
        return newArray
    }

    setNextDamage(player: Player, item: ItemStack, pos: Vector3, amount?: number){
        item.setDynamicProperty(`buidler_wand:old_damage_${item.typeId.replace("builder_wand:", "").replace("builder_", "")}`, amount ? JSON.stringify({damage: amount, pos: apiNumbers.vectorToString(pos)}) : undefined)
        player.getComponent("equippable").setEquipment("Mainhand", item)
    }

    testValidWand(item: ItemStack): boolean {
        return item.getTags().includes("builder_wand:wand") || item.getTags().includes("builder_wand:creative") ? true : false
    }

    testWandCreative(item: ItemStack): boolean {
        return item.getTags().includes("builder_wand:creative") ? true : false
    }
}

interface oldDamage {
    damage: number,
    pos: string
}